from easymql.core import Literal

ABS = Literal("ABS")
ACOS = Literal('ACOS')
ACOSH = Literal('ACOSH')
ADD = Literal("ADD")
ADD_TO_SET = Literal("ADD_TO_SET")
ALL = Literal('ALL')
ANY = Literal('ANY')
ARRAY_ELEM_AT = Literal("ARRAY_ELEM_AT")
ARRAY_TO_OBJECT = Literal("ARRAY_TO_OBJECT")
ASIN = Literal('ASIN')
ASINH = Literal('ASINH')
ATAN = Literal('ATAN')
ATAN2 = Literal('ATAN2')
ATANH = Literal('ATANH')
AVG = Literal("AVG")
BINARY_SIZE = Literal("BINARY_SIZE")
BSON_SIZE = Literal("BSON_SIZE")
CEIL = Literal("CEIL")
CMP = Literal("CMP")
CONCAT = Literal('CONCAT')
CONCAT_ARRAYS = Literal("CONCAT_ARRAYS")
CONVERT = Literal('CONVERT')
COS = Literal('COS')
DATE = Literal('DATE')
DEGREES_TO_RADIANS = Literal('DEGREES_TO_RADIANS')
DIVIDE = Literal("DIVIDE")
EXP = Literal("EXP")
EXTRACT = Literal('EXTRACT')
FILTER = Literal("FILTER")
FIRST = Literal("FIRST")
FLOOR = Literal("FLOOR")
FORMAT_DATE = Literal('FORMAT_DATE')
IF = Literal("IF")
IF_NULL = Literal("IF_NULL")
IN = Literal("IN")
INDEX_OF_ARRAY = Literal("INDEX_OF_ARRAY")
INDEX_OF_BYTES = Literal('INDEX_OF_BYTES')
INDEX_OF_CP = Literal('INDEX_OF_CP')
ISO_WEEK_DATE = Literal('ISO_WEEK_DATE')
IS_ARRAY = Literal("IS_ARRAY")
IS_NUMBER = Literal('IS_NUMBER')
LAST = Literal("LAST")
LN = Literal("LN")
LOG = Literal("LOG")
LOG10 = Literal("LOG10")
LTRIM = Literal('LTRIM')
MAP = Literal("MAP")
MAX = Literal("MAX")
MERGE_OBJECTS = Literal("MERGE_OBJECTS")
MIN = Literal("MIN")
MOD = Literal("MOD")
MULTIPLY = Literal("MULTIPLY")
OBJECT_TO_ARRAY = Literal("OBJECT_TO_ARRAY")
PARSE_DATE = Literal('PARSE_DATE')
POW = Literal("POW")
PUSH = Literal("PUSH")
RADIANS_TO_DEGREES = Literal('RADIANS_TO_DEGREES')
RANDOM = Literal("RANDOM")
RANGE = Literal("RANGE")
REDUCE = Literal("REDUCE")
REGEX_FIND = Literal('REGEX_FIND')
REGEX_FIND_ALL = Literal('REGEX_FIND_ALL')
REGEX_MATCH = Literal('REGEX_MATCH')
REPLACE = Literal('REPLACE')
REPLACE_ALL = Literal('REPLACE_ALL')
REVERSE_ARRAY = Literal("REVERSE_ARRAY")
ROUND = Literal("ROUND")
RTRIM = Literal('RTRIM')
SAMPLE_RATE = Literal("SAMPLE_RATE")
SET_DIFFERENCE = Literal('SET_DIFFERENCE')
SET_EQUALS = Literal('SET_EQUALS')
SET_INTERSECTION = Literal('SET_INTERSECTION')
SET_IS_SUBSET = Literal('SET_IS_SUBSET')
SET_UNION = Literal('SET_UNION')
SIN = Literal('SIN')
SIZE = Literal("SIZE")
SLICE = Literal("SLICE")
SPLIT = Literal('SPLIT')
SQRT = Literal("SQRT")
STD_DEV_POP = Literal("STD_DEV_POP")
STD_DEV_SAMP = Literal("STD_DEV_SAMP")
STR_CASE_CMP = Literal('STR_CASE_CMP')
STR_LEN_BYTES = Literal('STR_LEN_BYTES')
STR_LEN_CP = Literal('STR_LEN_CP')
SUBSTR = Literal('SUBSTR')
SUBSTR_BYTES = Literal('SUBSTR_BYTES')
SUBSTR_CP = Literal('SUBSTR_CP')
SUBTRACT = Literal("SUBTRACT")
SUM = Literal("SUM")
TAN = Literal('TAN')
TO_BOOL = Literal('TO_BOOL')
TO_DATE = Literal('TO_DATE')
TO_DECIMAL = Literal('TO_DECIMAL')
TO_DOUBLE = Literal('TO_DOUBLE')
TO_INT = Literal('TO_INT')
TO_LONG = Literal('TO_LONG')
TO_LOWER = Literal('TO_LOWER')
TO_OBJECT_ID = Literal('TO_OBJECT_ID')
TO_STRING = Literal('TO_STRING')
TO_UPPER = Literal('TO_UPPER')
TRIM = Literal('TRIM')
TRUNC = Literal("TRUNC")
TYPE = Literal('TYPE')
ZIP = Literal("ZIP")
