from easymql.core import Keyword

ADD = Keyword("ADD")
ARRAY = Keyword('ARRAY')
ARRAYS = Keyword('ARRAYS')
AS = Keyword('AS')
ASC = Keyword('ASC')
BOUNDARIES = Keyword('BOUNDARIES')
BUCKET = Keyword('BUCKET')
BY = Keyword('BY')
CASE = Keyword("CASE")
COLL = Keyword('COLL')
COUNT = Keyword('COUNT')
DB = Keyword("DB")
DEFAULT = Keyword('DEFAULT')
DESC = Keyword('DESC')
DISCARD = Keyword('DISCARD')
ELSE = Keyword("ELSE")
EMPTY = Keyword('EMPTY')
END = Keyword("END")
FACET = Keyword('FACET')
FAIL = Keyword('FAIL')
FIELDS = Keyword('FIELDS')
GROUP = Keyword('GROUP')
INDEX = Keyword('INDEX')
INSERT = Keyword('INSERT')
INTO = Keyword('INTO')
KEEP = Keyword('KEEP')
LIMIT = Keyword('LIMIT')
LOOKUP = Keyword('LOOKUP')
MATCH = Keyword('MATCH')
MATCHED = Keyword('MATCHED')
MERGE = Keyword('MERGE')
NOT = Keyword('NOT')
NULL = Keyword('NULL')
OFFSET = Keyword('OFFSET')
ON = Keyword('ON')
ORDER = Keyword('ORDER')
OUTPUT = Keyword('OUTPUT')
PIPELINE = Keyword('PIPELINE')
PRESERVE = Keyword('PRESERVE')
PROJECT = Keyword('PROJECT')
REDACT = Keyword('REDACT')
REPLACE = Keyword('REPLACE')
ROOT = Keyword('ROOT')
SAMPLE = Keyword('SAMPLE')
SET = Keyword('SET')
SKIP = Keyword('SKIP')
SORT = Keyword('SORT')
THEN = Keyword("THEN")
TO = Keyword('TO')
UNION = Keyword('UNION')
UNSET = Keyword('UNSET')
UNWIND = Keyword('UNWIND')
WHEN = Keyword("WHEN")
WITH = Keyword('WITH')
false = Keyword('false')
null = Keyword('null')
true = Keyword('true')

# System variables
SYS_CLUSTER_TIME = Keyword('$CLUSTER_TIME')
SYS_CURRENT = Keyword('$CURRENT')
SYS_DESCEND = Keyword('$DESCEND')
SYS_KEEP = Keyword('$KEEP')
SYS_NOW = Keyword('$NOW')
SYS_PRUNE = Keyword('$PRUNE')
SYS_REMOVE = Keyword('$REMOVE')
SYS_ROOT = Keyword('$ROOT')
